" Vim indent file
" Language:     luakit configuration
" Maintainer:   Gregor Uhlenheuer <kongo2002@googlemail.com>
" Last Change:  Fri 27 Aug 2010 09:49:20 PM CEST

if exists('b:did_indent')
    finish
endif

runtime! indent/lua.vim
