--- Spinner widget
--
-- DOCMACRO(available:ui)
--
-- The spinner widget indicates an ongoing process of indefinite duration.
--
-- @class widget:spinner
-- @prefix spinner
-- @author Mason Larobina
-- @copyright 2010 Mason Larobina <mason.larobina@gmail.com>

--- @method start
-- Start spinning.

--- @method stop
-- Stop spinning.

--- @property started
-- Whether the widget is spinning.
-- @type boolean
-- @readonly

-- vim: et:sw=4:ts=8:sts=4:tw=80
